MODULE crsdom
   !!===================================================================
   !!                  ***  crs.F90 ***
   !!  Purpose: Interface for calculating quantities from a  
   !!           higher-resolution grid for the coarse grid.
   !!
   !!  Method:  Given the user-defined reduction factor, 
   !!           the averaging bins are set: 
   !!           - cn_binref = NORTH, starting from the north 
   !!           to the south in the model interior domain, 
   !!           in this way the north fold and redundant halo cells  
   !!           could be handled in a consistent manner and 
   !!           the irregularities of bin size can be handled
   !!           more naturally by the presence of land
   !!           in the southern boundary.  Thus the southernmost bin 
   !!           could be of an irregular bin size.
   !!           Information on the parent grid is retained, specifically,
   !!           each coarse grid cell's volume and ocean surface 
   !!           at the faces, relative to the parent grid.
   !!           - cn_binref = EQUAT (not yet available), starting
   !!           at a centralized bin at the equator, being only
   !!           truly centered for odd-numbered j-direction reduction 
   !!           factors.
   !!  References:  Aumont, O., J.C. Orr, D. Jamous, P. Monfray
   !!               O. Marti and G. Madec, 1998. A degradation 
   !!               approach to accelerate simulations to steady-state 
   !!               in a 3-D tracer transport model of the global ocean.
   !!               Climate Dynamics, 14:101-116.
   !!  History: 
   !!       Original.   May 2012.  (J. Simeon, G. Madec, C. Ethe, C. Calone)
   !!===================================================================

   USE dom_oce        ! ocean space and time domain and to get jperio
   USE wrk_nemo       ! work arrays
   USE crs            ! domain for coarse grid
   USE in_out_manager 
   USE par_kind, ONLY: wp
   USE crslbclnk
   

   IMPLICIT NONE

   PRIVATE 

   PUBLIC crsfun, crsfun_wgt
   PUBLIC crs_dom_e3_max, crs_dom_sfc, crs_dom_msk, crs_dom_def, crs_dom_hgr, crs_dom_coordinates, crs_dom_bat

   INTERFACE crsfun
      MODULE PROCEDURE crsfun_UV, crsfun_TW
   END INTERFACE

   !! Substitutions
#  include "domzgr_substitute.h90"
   
CONTAINS


   SUBROUTINE crs_dom_msk
      
      INTEGER  ::  ji, jj, jk, ijpk                   ! dummy loop indices
      INTEGER  :: ijie,ijis,ijje,ijjs
      REAL(wp) ::   zmask
      
      ! Initialize

      tmask_crs(:,:,:) = 0.0
      vmask_crs(:,:,:) = 0.0
      umask_crs(:,:,:) = 0.0
      fmask_crs(:,:,:) = 0.0
      
      DO jk = 1, jpk
         DO ji = 2, nlei_crs  
            ijie = mie_crs(ji)
            ijis = mis_crs(ji)
            DO jj = njstart, njend 
               ijje = mje_crs(jj) 
               ijjs = mjs_crs(jj) 
                                 
               zmask = 0
               zmask = SUM( tmask(ijis:ijie,ijjs:ijje,jk) ) 
               IF ( zmask > 0 ) tmask_crs(ji,jj,jk) = 1
               
               zmask = 0
               zmask = SUM( vmask(ijis:ijie,ijje     ,jk) )  
               IF ( zmask > 0 ) vmask_crs(ji,jj,jk) = 1
               
               zmask = 0
               zmask = SUM( umask(ijie     ,ijjs:ijje,jk) )  
               IF ( zmask > 0 ) umask_crs(ji,jj,jk) = 1
               
               fmask_crs(ji,jj,jk) = fmask(ijie,ijje,jk)  

            ENDDO
         ENDDO
      ENDDO
      !
      CALL crs_lbc_lnk( tmask_crs(:,:,:), 'T', psgn = 1.0 )
      CALL crs_lbc_lnk( vmask_crs(:,:,:), 'V', psgn = 1.0 )
      CALL crs_lbc_lnk( umask_crs(:,:,:), 'U', psgn = 1.0 )
      CALL crs_lbc_lnk( fmask_crs(:,:,:), 'F', psgn = 1.0 )
      !
   END SUBROUTINE crs_dom_msk

   SUBROUTINE crs_dom_coordinates( p_pgphi, p_pglam, cd_type, p_cgphi, p_cglam )
      !!----------------------------------------------------------------
      !!               *** SUBROUTINE crs_coordinates ***
      !! ** Purpose :  Determine the coordinates for the coarse grid
      !! 
      !! ** Method  :  From the parent grid subset, search for the central
      !!               point.  For an odd-numbered reduction factor,
      !!               the coordinate will be that of the central T-cell.
      !!               For an even-numbered reduction factor, of a non-square
      !!               coarse grid box, the coordinate will be that of 
      !!               the east or north face or more likely.  For a square
      !!               coarse grid box, the coordinate will be that of
      !!               the central f-corner.
      !!
      !! ** Input   :  p_pgphi = parent grid gphi[t|u|v|f]
      !!               p_pglam = parent grid glam[t|u|v|f]
      !!               cd_type  = grid type (T,U,V,F)
      !! ** Output  :  p_cgphi = coarse grid gphi[t|u|v|f]
      !!               p_cglam = coarse grid glam[t|u|v|f]
      !!              
      !! History. 1 Jun.
      !!----------------------------------------------------------------
      !! Arguments
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in)  :: p_pgphi  ! Parent grid latitude
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in)  :: p_pglam  ! Parent grid longitude
      CHARACTER(len=1),             INTENT(in)  :: cd_type   ! grid type (T,U,V,F) 
      REAL(wp), DIMENSION(jpi_crs,jpj_crs), INTENT(out) :: p_cgphi  ! Coarse grid latitude
      REAL(wp), DIMENSION(jpi_crs,jpj_crs), INTENT(out) :: p_cglam  ! Coarse grid longitude

      !! Local variables
      INTEGER ::  ji, jj, jk                   ! dummy loop indices
      INTEGER :: ijie,ijis,ijje,ijjs,ijpk

  
      !! Initialize output fields
      p_cgphi(:,:) = 0.e0
      p_cglam(:,:) = 0.e0

       DO ji = 2, nlei_crs

         IF ( cd_type == 'T' .OR. cd_type == 'V' )  ijis = mis_crs(ji) + mxbinctr 
         IF ( cd_type == 'U' .OR. cd_type == 'F' )  ijis = mie_crs(ji)

         DO jj = njstart, njend
     
            IF ( cd_type == 'T' .OR. cd_type == 'U' ) ijjs = mjs_crs(jj) + mybinctr                  
            IF ( cd_type == 'V' .OR. cd_type == 'F' ) ijjs = mje_crs(jj)

            p_cgphi(ji,jj) = p_pgphi(ijis,ijjs)
            p_cglam(ji,jj) = p_pglam(ijis,ijjs)
             
         ENDDO

      ENDDO


       ! Retroactively add back the boundary halo cells.

         CALL crs_lbc_lnk( p_cgphi(:,:),cd_type,1.0 )
         CALL crs_lbc_lnk( p_cglam(:,:),cd_type,1.0 )
         
      ! Fill up jrow=1 which is zeroed out or not handled by lbc_lnk and lbc_nfd

    DO ji = 2, nlei_crs

         IF ( cd_type == 'T' .OR. cd_type == 'V' )  ijis = mis_crs(ji) + mxbinctr 
         IF ( cd_type == 'U' .OR. cd_type == 'F' )  ijis = mie_crs(ji)

         p_cgphi(ji,1) = p_pgphi(ijis,1)
         p_cglam(ji,1) = p_pglam(ijis,1)

      ENDDO

      ! Fill i=1, i=jpi at j=1
      p_cgphi(1,1) = p_cgphi(jpi_crsm1,1)
      p_cglam(1,1) = p_cglam(jpi_crsm1,1)
      p_cgphi(jpi_crs,1) = p_cgphi(2,1)
      p_cglam(jpi_crs,1) = p_cglam(2,1)
      ! Fill upper-right corner i=1, j=jpj_crs
 
   END SUBROUTINE crs_dom_coordinates

  SUBROUTINE crs_dom_hgr( p_e1, p_e2, cd_type, p_e1_crs, p_e2_crs )
      !!----------------------------------------------------------------
      !!               *** SUBROUTINE crs_dom_hgr ***
      !!
      !! ** Purpose :  Get coarse grid horizontal scale factors and unmasked fraction
      !!
      !! ** Method  :  For grid types T,U,V,Fthe 2D scale factors of 
      !!               the coarse grid are the sum of the east or north faces of the 
      !!               parent grid subset comprising the coarse grid box.     
      !!               - e1,e2 Scale factors
      !!                 Valid arguments: 
      !! ** Inputs  : p_e1, p_e2  = parent grid e1 or e2 (t,u,v,f)
      !!              cd_type     = grid type (T,U,V,F) for scale factors; for velocities (U or V)
      !! ** Outputs : p_e1_crs, p_e2_crs  = parent grid e1 or e2 (t,u,v,f)
      !!
      !! History.     4 Jun.  Write for WGT and scale factors only
      !!----------------------------------------------------------------
      !! 
      !!  Arguments
      REAL(wp), DIMENSION(jpi,jpj)        , INTENT(in)  :: p_e1     ! Parent grid U,V scale factors (e1)
      REAL(wp), DIMENSION(jpi,jpj)        , INTENT(in)  :: p_e2     ! Parent grid U,V scale factors (e2)
      CHARACTER(len=1)                    , INTENT(in)  :: cd_type  ! grid type U,V 

      REAL(wp), DIMENSION(jpi_crs,jpj_crs), INTENT(out) :: p_e1_crs ! Coarse grid box 2D quantity
      REAL(wp), DIMENSION(jpi_crs,jpj_crs), INTENT(out) :: p_e2_crs ! Coarse grid box 2D quantity

      !! Local variables
      INTEGER                                 :: ji, jj, jk     ! dummy loop indices
      INTEGER                                 :: ijie,ijis,ijje,ijjs,ijrs
  
      !!----------------------------------------------------------------  
      ! Initialize      

      DO jk = 1, jpk    
         DO ji = 2, nlei_crs
            ijie = mie_crs(ji)
            ijis = mis_crs(ji)
            DO jj = njstart, njend
               ijje = mje_crs(jj) 
               ijjs = mjs_crs(jj)                   
               ijrs =  mje_crs(jj) - mjs_crs(jj)
               ! Only for a factro 3 coarsening
               SELECT CASE ( cd_type )
                   CASE ( 'T' )
                      IF( ijrs == 0 .OR. ijrs == 1 ) THEN
                        ! Si à la frontière sud on a pas assez de maille de la grille mère
                         p_e1_crs(ji,jj) = p_e1(ijie-1,ijje) * nn_factx
                         p_e2_crs(ji,jj) = p_e2(ijie-1,ijje) * nn_facty
                      ELSE
                         p_e1_crs(ji,jj) = p_e1(ijie-1,ijje-1) * nn_factx
                         p_e2_crs(ji,jj) = p_e2(ijie-1,ijje-1) * nn_facty
                      ENDIF
                   CASE ( 'U' )
                      IF( ijrs == 0 .OR. ijrs == 1 ) THEN
                         ! Si à la frontière sud on a pas assez de maille de la grille mère
                         p_e1_crs(ji,jj) = p_e1(ijie,ijje) * nn_factx                            
                         p_e2_crs(ji,jj) = p_e2(ijie,ijje) * nn_facty
                      ELSE
                         p_e1_crs(ji,jj) = p_e1(ijie,ijje-1) * nn_factx
                         p_e2_crs(ji,jj) = p_e2(ijie,ijje-1) * nn_facty
                      ENDIF
                   CASE ( 'V' )
                         p_e1_crs(ji,jj) = p_e1(ijie-1,ijje) * nn_factx                            
                         p_e2_crs(ji,jj) = p_e2(ijie-1,ijje) * nn_facty
                   CASE ( 'F' )
                         p_e1_crs(ji,jj) = p_e1(ijie,ijje) * nn_factx                            
                         p_e2_crs(ji,jj) = p_e2(ijie,ijje) * nn_facty
               END SELECT
            ENDDO
         ENDDO
      ENDDO

      CALL crs_lbc_lnk( p_e1_crs, cd_type, 1.0, pval=1.0 )
      CALL crs_lbc_lnk( p_e2_crs, cd_type, 1.0, pval=1.0 )

   END SUBROUTINE crs_dom_hgr


   SUBROUTINE crsfun_wgt( cd_type, cd_op, p_pmask, p_e1, p_e2, p_fse3, &
      &                   p_cfield2d_1, p_cfield2d_2, p_cfield3d_1, p_cfield3d_2 )
      !!----------------------------------------------------------------
      !!               *** SUBROUTINE crsfun_wgt ***
      !! ** Purpose :  Three applications.
      !!               1) SUM. Get coarse grid horizontal scale factors and unmasked fraction
      !!               2) VOL. Get coarse grid box volumes
      !!               3) WGT. Weighting multiplier for volume-weighted and/or
      !!                       area-weighted averages.
      !!                       Weights (i.e. the denominator) calculated here 
      !!                       to avoid IF-tests and division.
      !! ** Method  :  1) SUM.  For grid types T,U,V,F (and W) the 2D scale factors of 
      !!               the coarse grid are the sum of the east or north faces of the 
      !!               parent grid subset comprising the coarse grid box.  
      !!               The fractions of masked:total surface (3D) on the east, 
      !!               north and top faces is, optionally, also output.
      !!               - Top face area sum
      !!                 Valid arguments: cd_type, cd_op='W', p_pmask, p_e1, p_e2 
      !!               - Top face ocean surface fraction
      !!                 Valid arguments: cd_type, cd_op='W', p_pmask, p_e1, p_e2       
      !!               - e1,e2 Scale factors
      !!                 Valid arguments: 
      !!               2) VOL.  For grid types W and T, the coarse grid box
      !!               volumes are output. Also optionally, the fraction of  
      !!               masked:total volume of the parent grid subset is output (i.e. facvol).
      !!               3) WGT. Based on the grid type, the denominator is pre-determined here to  
      !!               perform area- or volume- weighted averages, 
      !!               to avoid IF-tests and divisions.
      !! ** Inputs  : p_e1, p_e2  = parent grid e1 or e2 (t,u,v,f)
      !!              p_pmask     = parent grid mask (T,U,V,F) 
      !!              cd_type     = grid type (T,U,V,F) for scale factors; for velocities (U or V)
      !!              cd_op       = applied operation (SUM, VOL, WGT)
      !!              p_fse3      = (Optional) parent grid vertical level thickness (fse3u or fse3v)
      !! ** Outputs : p_cfield2d_1 = (Optional) 2D field on coarse grid
      !!              p_cfield2d_2 = (Optional) 2D field on coarse grid
      !!              p_cfield3d_1 = (Optional) 3D field on coarse grid
      !!              p_cfield3d_2 = (Optional) 3D field on coarse grid
      !!
      !! History.     4 Jun.  Write for WGT and scale factors only
      !!----------------------------------------------------------------
      !! 
      !!  Arguments
      CHARACTER(len=1),                 INTENT(in) :: cd_type  ! grid type U,V 
      CHARACTER(len=3),                 INTENT(in) :: cd_op    ! operation sum or average
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in) :: p_pmask  ! Parent grid U,V mask
      REAL(wp), DIMENSION(jpi,jpj),     INTENT(in) :: p_e1     ! Parent grid U,V scale factors (e1)
      REAL(wp), DIMENSION(jpi,jpj),     INTENT(in) :: p_e2     ! Parent grid U,V scale factors (e2)

      REAL(wp), DIMENSION(:,:),   INTENT(out), OPTIONAL :: p_cfield2d_1 ! Coarse grid box 2D quantity
      REAL(wp), DIMENSION(:,:),   INTENT(out), OPTIONAL :: p_cfield2d_2 ! Coarse grid box 2D quantity
      REAL(wp), DIMENSION(:,:,:), INTENT(out), OPTIONAL :: p_cfield3d_1 ! Coarse grid box 3D quantity 
      REAL(wp), DIMENSION(:,:,:), INTENT(out), OPTIONAL :: p_cfield3d_2 ! Coarse grid box 3D quantity 

      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in), OPTIONAL :: p_fse3     ! Parent grid vertical level thickness (fse3u, fse3v)

      !! Local variables
      INTEGER                                 ::  ji, jj, jk, jii, jjj     ! dummy loop indices
      INTEGER                                 :: ijie,ijis,ijje,ijjs,ijpk
      REAL(wp)                                :: zdAm                      ! masked face area
      REAL(wp), DIMENSION(:,:),   POINTER     :: ze1, ze2
      REAL(wp), DIMENSION(:,:,:), POINTER     :: ze3       
      REAL(wp), DIMENSION(:,:),   POINTER     :: zcfield2d_1, zcfield2d_2
      REAL(wp), DIMENSION(:,:,:), POINTER     :: zcfield3d_1, zcfield3d_2
  
      !!----------------------------------------------------------------  
      ! Initialize      

      ! Arrays, scalars initialization 
      CALL wrk_alloc(jpi    , jpj         , ze1, ze2 )
      CALL wrk_alloc(jpi    , jpj    , jpk, ze3 )
      CALL wrk_alloc(jpi_crs, jpj_crs,      zcfield2d_1,  zcfield2d_2 )
      CALL wrk_alloc(jpi_crs, jpj_crs, jpk, zcfield3d_1,  zcfield3d_2 )

      ze1(:,:) = 1.0
      ze2(:,:) = 1.0
      ze3(:,:,:) = 1.0
      zcfield2d_1(:,:)  = 0.0
      zcfield2d_2(:,:)  = 0.0
      zcfield3d_1(:,:,:) = 0.0
      zcfield3d_2(:,:,:) = 0.0
    
      ijpk = jpk

     ! Control of arguments
      ze1(:,:) = p_e1(:,:)
      ze2(:,:) = p_e2(:,:)

      IF ( PRESENT(p_cfield2d_1) ) p_cfield2d_1(:,:) = 0.0
      IF ( PRESENT(p_cfield2d_2) ) p_cfield2d_2(:,:) = 0.0
      IF ( PRESENT(p_cfield3d_1) ) p_cfield3d_1(:,:,:) = 0.0
      IF ( PRESENT(p_cfield3d_2) ) p_cfield3d_2(:,:,:) = 0.0

      IF ( PRESENT(p_fse3) ) ze3(:,:,:) = p_fse3(:,:,:)


          DO jk = 1, ijpk    

             zcfield2d_1(:,:) = 0.0 ; zcfield2d_2(:,:) = 0.0        
            DO ji = 2, nlei_crs
                ijie = mie_crs(ji)
                ijis = mis_crs(ji)


                DO jj = njstart, njend
                   ijje = mje_crs(jj) 
                   ijjs = mjs_crs(jj)                   
                   
                   IF ( cd_op == 'POS' ) THEN      !cc
                   
                      IF ( nn_factx == 3 .AND. nn_facty == 3) THEN
                      
                         SELECT CASE ( cd_type )
                         
                            CASE ( 'T' )

                               SELECT CASE ( mje_crs(jj)-mjs_crs(jj) )
                                     
                                  CASE( 0, 1 )    ! Si à la frontière sud on a pas assez de maille de la grille mère
                                      
                                     zcfield2d_1(ji,jj) = ze1(ijie-1,ijje  ) * nn_factx
                                     zcfield2d_2(ji,jj) = ze2(ijie-1,ijje  ) * nn_facty
                                       
                                  CASE DEFAULT
   
                                     zcfield2d_1(ji,jj) = ze1(ijie-1,ijje-1) * nn_factx
                                     zcfield2d_2(ji,jj) = ze2(ijie-1,ijje-1) * nn_facty
 
                                  END SELECT
                             
                            CASE ( 'U' )
                            
                               SELECT CASE ( mje_crs(jj)-mjs_crs(jj) )
                               
                                  CASE( 0, 1 )    ! Si à la frontière sud on a pas assez de maille de la grille mère
                                  
                                     zcfield2d_1(ji,jj) = ze1(ijie  ,ijje  ) * nn_factx                            
                                     zcfield2d_2(ji,jj) = ze2(ijie  ,ijje  ) * nn_facty
                                     
                                  CASE DEFAULT
                                     
                                     zcfield2d_1(ji,jj) = ze1(ijie  ,ijje-1) * nn_factx                            
                                     zcfield2d_2(ji,jj) = ze2(ijie  ,ijje-1) * nn_facty
                                     
                               END SELECT
                                
                            CASE ( 'V' )

                               zcfield2d_1(ji,jj) = ze1(ijie-1,ijje  ) * nn_factx
                               zcfield2d_2(ji,jj) = ze2(ijie-1,ijje  ) * nn_facty
                                                              
                            CASE ( 'F' )

                               zcfield2d_1(ji,jj) = ze1(ijie  ,ijje  ) * nn_factx                         
                               zcfield2d_2(ji,jj) = ze2(ijie  ,ijje  ) * nn_facty
                               
                         END SELECT
                      ENDIF
                   ENDIF                    
                   

                   IF ( cd_op == 'WGT' ) THEN

                      zdAm = 0.0

                      IF ( cd_type == 'V' ) THEN
                         ! 
                         DO jii = ijis, ijie
                            zdAm = zdAm + ( ze1(jii,ijje) * ze3(jii,ijje,jk) * p_pmask(jii,ijje,jk) ) 
                         ENDDO
                         IF ( zdAm /= 0 ) zcfield3d_1(ji,jj,jk) = zdAm 
    
                      ELSEIF ( cd_type == 'U') THEN
                         DO jjj = ijjs, ijje
                            zdAm = zdAm + ( ze2(ijie,jjj) * ze3(ijie,jjj,jk) * p_pmask(ijie,jjj,jk) )
                         ENDDO
                         IF ( zdAm /= 0 ) zcfield3d_1(ji,jj,jk) = zdAm 

                      ELSEIF ( cd_type == 'W' ) THEN
                         DO jii = ijis, ijie
                            DO jjj = ijjs, ijje
                               zdAm = zdAm + ( ze1(jii,jjj) * ze2(jii,jjj) * p_pmask(jii,jjj,jk) )
                            ENDDO
                         ENDDO
                         IF ( zdAm /= 0 ) zcfield3d_1(ji,jj,jk) = zdAm

                      ELSEIF ( cd_type == 'T' ) THEN
                         DO jii = ijis, ijie
                            DO jjj = ijjs, ijje
                               zdAm = zdAm + ( ze1(jii,jjj) * ze2(jii,jjj) * ze3(jii,jjj,jk) * p_pmask(jii,jjj,jk) )
                            ENDDO
                         ENDDO
                         IF ( zdAm /= 0 ) zcfield3d_1(ji,jj,jk) = zdAm

                      ELSE
  
                         ! jes. Add a stop?

                      ENDIF

                   ENDIF

                   IF ( cd_op == 'VOL' ) THEN

                      zdAm = 0.0

                      IF ( cd_type == 'W' .OR. cd_type == 'T' ) THEN

                         DO jii = ijis, ijie
                            DO jjj = ijjs, ijje
                               zcfield3d_1(ji,jj,jk) = zcfield3d_1(ji,jj,jk) + ( ze1(jii,jjj) * ze2(jii,jjj) * ze3(jii,jjj,jk) )
                               zdAm = zdAm + ( ze1(jii,jjj) * ze2(jii,jjj) * ze3(jii,jjj,jk) * p_pmask(jii,jjj,jk) )
                            ENDDO
                         ENDDO
                         IF ( zcfield3d_1(ji,jj,jk) /= 0 ) zcfield3d_2(ji,jj,jk) = zdAm / zcfield3d_1(ji,jj,jk)

                      ELSE
                          ! jes. add a stop?
                      ENDIF

                   ENDIF
                                    
                ENDDO
             ENDDO

          ENDDO

! Retroactively add back the boundary halo cells.



         ! Take care of the 2D arrays
         IF ( cd_op == 'SUM' .OR.  cd_op == 'POS') THEN
            IF ( PRESENT(p_cfield2d_1) ) THEN
               p_cfield2d_1(:,:) = zcfield2d_1(:,:)
               CALL crs_lbc_lnk( p_cfield2d_1(:,:),cd_type,1.0, pval=1.0)
            ENDIF
            
            IF ( PRESENT(p_cfield2d_2) ) THEN
               p_cfield2d_2(:,:) = zcfield2d_2(:,:)
               CALL crs_lbc_lnk( p_cfield2d_2(:,:),cd_type,1.0, pval=1.0 )
               
               IF ( cd_op == 'SUM') THEN 
                  DO jii = 1 , jpiglo_crs
                  p_cfield2d_2(jii,1) = p_cfield2d_2(jii,1) * 3
                  ENDDO
               ENDIF
            ENDIF

         ELSE

            CALL crs_lbc_lnk( zcfield2d_1(:,:),cd_type,1.0 ) 
            IF ( PRESENT(p_cfield2d_1) ) p_cfield2d_1(:,:) = zcfield2d_1(:,:)
            CALL crs_lbc_lnk( zcfield2d_2(:,:),cd_type,1.0 ) 
            IF ( PRESENT(p_cfield2d_2) ) p_cfield2d_2(:,:) = zcfield2d_2(:,:)

         ENDIF

         ! Take care now of 3d arrays
         IF ( cd_op == 'SUM' .OR. cd_op == 'VOL' .OR. cd_op == 'POS'  ) THEN
            CALL crs_lbc_lnk( zcfield3d_1(:,:,:),cd_type,1.0 ) 
            IF ( PRESENT(p_cfield3d_1) ) p_cfield3d_1(:,:,:) = zcfield3d_1(:,:,:)
            CALL crs_lbc_lnk( zcfield3d_2(:,:,:),cd_type,1.0 ) 
            IF ( PRESENT(p_cfield3d_2) ) p_cfield3d_2(:,:,:) = zcfield3d_2(:,:,:)
         ELSE
            p_cfield3d_1(:,:,:) = zcfield3d_1(:,:,:)
            CALL crs_lbc_lnk( p_cfield3d_1(:,:,:),cd_type,1.0 )

            ! Fill upper-right corner i=1, j=jpj_crs
            IF ( nperio == 4 ) THEN
               p_cfield3d_1(1,jpj_crs,:) = p_cfield3d_1(jpi_crsm1,jpj_crs-2,:)
            ELSEIF ( nperio == 6 ) THEN
               p_cfield3d_1(1,jpj_crs,:) = p_cfield3d_1(jpi_crs,jpj_crsm1,:)
            ENDIF

         ENDIF

      CALL wrk_dealloc(jpi    , jpj         , ze1, ze2 )
      CALL wrk_dealloc(jpi    , jpj    , jpk, ze3 )
      CALL wrk_dealloc(jpi_crs, jpj_crs,      zcfield2d_1,  zcfield2d_2 )
      CALL wrk_dealloc(jpi_crs, jpj_crs, jpk, zcfield3d_1,  zcfield3d_2 )

   END SUBROUTINE crsfun_wgt


   SUBROUTINE crsfun_UV( p_e1_e2, cd_type, psgn, p_pmask, p_fse3, p_pfield, p_surf_crs, p_cfield3d )
      !!----------------------------------------------------------------
      !!               *** SUBROUTINE crsfun_UV ***
      !! ** Purpose :  Average, area-weighted, of U or V on the east and north faces 
      !!
      !! ** Method  :  The U and V velocities (3D) are determined as the area-weighted averages
      !!               on the east and north faces, respectively,
      !!               of the parent grid subset comprising the coarse grid box. 
      !!               In the case of the V and F grid, the last jrow minus 1 is spurious.
      !! ** Inputs  : p_e1_e2     = parent grid e1 or e2 (t,u,v,f)
      !!              cd_type     = grid type (T,U,V,F) for scale factors; for velocities (U or V)
      !!              psgn        = sign change over north fold (See lbclnk.F90)
      !!              p_pmask     = parent grid mask (T,U,V,F) for scale factors; 
      !!                                       for velocities (U or V)
      !!              p_fse3      = parent grid vertical level thickness (fse3u or fse3v)
      !!              p_pfield    = U or V on the parent grid
      !!              p_surf_crs  = (Optional) Coarse grid weight for averaging
      !! ** Outputs : p_cfield3d = 3D field on coarse grid
      !!
      !! History.  29 May.  completed draft.
      !!            4 Jun.  Revision for WGT
      !!            5 Jun.  Streamline for area-weighted average only ; separate scale factor and weights.
      !!----------------------------------------------------------------
      !! 
      !!  Arguments
      REAL(wp), DIMENSION(jpi,jpj),     INTENT(in)     :: p_e1_e2    ! Parent grid U,V scale factors (e1 or e2)
      CHARACTER(len=1),                 INTENT(in)     :: cd_type    ! grid type U,V 
      REAL(wp),                         INTENT(in)     :: psgn       ! sign change option across north fold
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in)     :: p_pmask    ! Parent grid U,V mask
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in)     :: p_fse3     ! Parent grid vertical level thickness (fse3u, fse3v)
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in)     :: p_pfield   ! U or V on parent grid
      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(in), OPTIONAL :: p_surf_crs ! Coarse grid area-weighting denominator    

      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(out)  :: p_cfield3d ! Coarse grid box 3D quantity 

      !! Local variables
      INTEGER  :: ji, jj, jk , jii, jjj                  ! dummy loop indices
      INTEGER  :: ijie, ijis, ijje, ijjs
      REAL(wp), DIMENSION(:,:,:), POINTER :: zsurfcrs   

      !!----------------------------------------------------------------  

      CALL wrk_alloc( jpi_crs, jpj_crs, jpk, zsurfcrs )
      zsurfcrs(:,:,:) = 1.0
      IF ( PRESENT(p_surf_crs) ) THEN
         WHERE ( p_surf_crs /= 0 ) zsurfcrs(:,:,:) = 1.0/p_surf_crs(:,:,:)
      ENDIF

      DO jk = 1, jpk    

        DO ji = 2, nlei_crs
            ijie = mie_crs(ji)
            ijis = mis_crs(ji)

            DO jj = njstart, njend
               ijje = mje_crs(jj) 
               ijjs = mjs_crs(jj)                   

               IF ( cd_type == 'V' ) THEN

                  DO jii = ijis, ijie
                     p_cfield3d(ji,jj,jk) = p_cfield3d(ji,jj,jk) &
                       & + ( p_pfield(jii,ijje,jk) * p_e1_e2(jii,ijje) * p_fse3(jii,ijje,jk) * p_pmask(jii,ijje,jk) ) 
                  ENDDO
                  p_cfield3d(ji,jj,jk) = p_cfield3d(ji,jj,jk) * zsurfcrs(ji,jj,jk) 

               ELSEIF ( cd_type == 'U') THEN

                  DO jjj = ijjs, ijje
                     p_cfield3d(ji,jj,jk) = p_cfield3d(ji,jj,jk) &
                       & + ( p_pfield(ijie,jjj,jk) * p_e1_e2(ijie,jjj) * p_fse3(ijie,jjj,jk) * p_pmask(ijie,jjj,jk) )
                  ENDDO
                  p_cfield3d(ji,jj,jk) = p_cfield3d(ji,jj,jk) * zsurfcrs(ji,jj,jk)

               ENDIF
                 
            ENDDO
         ENDDO
      ENDDO

! Retroactively add back the boundary halo cells.

      CALL crs_lbc_lnk( p_cfield3d(:,:,:),cd_type,psgn )
      
      CALL wrk_dealloc( jpi_crs, jpj_crs, jpk, zsurfcrs )

   END SUBROUTINE crsfun_UV

   SUBROUTINE crsfun_TW( p_e1e2t, cd_type, cd_op, p_cmask, p_ptmask, psgn, p_pfield2d, p_pfield3d_1, p_pfield3d_2, &
      &                  p_cfield2d, p_cfield3d)
      !!----------------------------------------------------------------
      !!               *** SUBROUTINE crsfun_TW ***
      !! ** Purpose :  Five applications.
      !!               1) Maximum surface quantity 
      !!                  - Vertical scale factors (fse3t or fse3w) 
      !!                    max thickness of the parent grid for coarse grid scale factors.
      !!                  - or diffusion test
      !!               2) Area-weighted mean quantity: w, or other 3D or 2D quantity
      !!               3) Volume-weighted mean quantity: tracer
      !!               4) Minimum surface quantity (diffusion test)
      !!               5) Area- or Volume- weighted sum.
      !! ** Method  :  1) - cd_op = 'MAX'. Determines the max vertical thickness of grid boxes
      !!                    including partial steps for at the bottom
      !!                    for the coarsened grid, where within the subset of 
      !!                    the parent grid cells the maximum thickness is taken.
      !!                    Valid arguments: p_e1e2t, cd_type, cd_op, p_cmask, p_pfield3d_1
      !!                    Where, normally p_pfield3d_1 is e3t.
      !!                  - cd_op = 'MAX'. May also be used for say, determining the maximum value of Kz, 
      !!                    thus p_pfield3d_1 is set to the 3D field, Kz.
      !!                    Valid arguments: p_e1e2t, cd_type, cd_op, p_cmask, p_pfield3d_1
      !!               2) - cd_op = 'ARE'. Calculate the area-weighted average (surface e1t*e2t)  
      !!                    of vertical velocity, w.
      !!                    Valid arguments: p_e1e2t, cd_type, cd_op, p_cmask, p_pfield3d_1
      !!                  - cd_op = 'ARE'. Calculate area-weighted average of a 2D quantity (e.g. emp)
      !!                    Valid arguments: p_e1e2t, cd_type, cd_op, p_cmask, p_pfield2d
      !!               3) - cd_op = 'VOL'. Calculate the ocean volume (e1e2t*[fse3t|fse3w]) 
      !!                    Valid arguments: p_e1e2t, cd_type, cd_op, p_cmask, p_pfield3d_1
      !!                  - cd_op = 'VOL'. Calculate volume-weighted average (volume e1t*e2t*fse3t) of a quantity.
      !!                    Valid arguments: p_e1e2t, cd_type, cd_op, p_cmask, p_pfield3d_1, p_pfield3d_2
      !!               4) - cd_op = 'MIN'. Calculate the minimum value on surface e1t*e2t for 3D variables
      !!                  Valid arguments: p_e1e2t, cd_type, cd_op, p_cmask, p_pfield3d_1
      !!               5) - cd_op = 'SUM'. Calculate a dimesionally-weighted sum.  This could be area-weighted
      !!                  or volume-weighted sum. 
      !! ** Inputs  : p_e1e2t      = parent grid top face surface area, e1t*e2t 
      !!              cd_type      = grid type T, W (U, V, F) 
      !!              cd_op        = MAX, ARE, VOL, MIN, SUM
      !!              p_cmask      =  coarse grid mask
      !!              p_ptmask     =  parent grid tmask     
      !!              psgn         = (Optional) sign for lbc_lnk  
      !!              p_pfield2d   = (Optional) 2D field on parent grid
      !!              p_pfield3d_1 = (Optional) parent grid fse3t or fse3w
      !!              p_pfield3d_2 = (Optional) 3D field on parent grid
      !! ** Outputs : p_cfield2d   = (Optional) 2D field on coarse grid
      !!              p_cfield3d   = (Optional) 3D field on coarse grid
      !!
      !! 
      !! History.  30 May.  Editing.  To decide later: Keep all functionality or separate out the mean function.
      !!            7 Jun   TODO. Need to fix up the parent grid mask to optional like crsfun_UV...
      !!----------------------------------------------------------------
      !! 
      !!  Arguments
      REAL(wp), DIMENSION(jpi,jpj),               INTENT(in) :: p_e1e2t      ! Parent grid T surface (e1*e2)
      CHARACTER(len=1),                           INTENT(in) :: cd_type      ! grid type T, W ( U, V, F)
      CHARACTER(len=3),                           INTENT(in) :: cd_op        ! operation max, min, area-average, volume-average
      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk),   INTENT(in) :: p_cmask      ! Coarse grid T mask
      REAL(wp), DIMENSION(jpi,jpj,jpk),           INTENT(in) :: p_ptmask     ! Parent grid T mask
      REAL(wp),                         OPTIONAL, INTENT(in) :: psgn         ! sign for lbc_lnk
      REAL(wp), DIMENSION(jpi,jpj),     OPTIONAL, INTENT(in) :: p_pfield2d   ! 2D quantity on parent grid, e.g. ssh
      REAL(wp), DIMENSION(jpi,jpj,jpk), OPTIONAL, INTENT(in) :: p_pfield3d_1 ! Normally parent grid vertical level thickness
      REAL(wp), DIMENSION(jpi,jpj,jpk), OPTIONAL, INTENT(in) :: p_pfield3d_2 ! 3D tracer or W on parent grid

      REAL(wp), DIMENSION(jpi_crs,jpj_crs),     OPTIONAL, INTENT(out):: p_cfield2d ! Coarse grid box east or north face quantity
      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk), OPTIONAL, INTENT(out):: p_cfield3d ! Coarse grid box east or north face quantity 

      !! Local variables
      INTEGER ::  ji, jj, jk                   ! dummy loop indices
      INTEGER :: ijie,ijis,ijje,ijjs,ijpk,jii,jjj
      INTEGER, DIMENSION(3) :: idims
      REAL(wp), POINTER, DIMENSION(:,:)   :: ze1e2, zpfield2d, zcfield2d
      REAL(wp), POINTER, DIMENSION(:,:,:) :: ze3, zpfield3d, zcfield3d, zcmask, zpmask  
      REAL(wp)                            :: zdAm, zsgn
      !!----------------------------------------------------------------  
      ! Initialize

      CALL wrk_alloc(jpi    , jpj         , ze1e2, zpfield2d )
      CALL wrk_alloc(jpi    , jpj    , jpk,  ze3 , zpfield3d, zpmask )
      CALL wrk_alloc(jpi_crs, jpj_crs,      zcfield2d )
      CALL wrk_alloc(jpi_crs, jpj_crs, jpk, zcfield3d,  zcmask )


      ! Arrays, scalars initialization 
      zpfield2d(:,:)   = 0.0
      zpfield3d(:,:,:) = 0.0
      zcfield2d(:,:)   = 0.0
      zcfield3d(:,:,:) = 0.0
      zpmask(:,:,:)    = 1.0
      idims(:)         = 1

      zcmask(:,:,:)    = p_cmask(:,:,:)     
      zpmask(:,:,:)    = p_ptmask(:,:,:)

      ijpk             = jpk


      ! Control of optional arguments
      !
      IF ( PRESENT(psgn) ) THEN
         zsgn = psgn
      ELSE
         zsgn = 1.0 
      ENDIF
      !
      IF ( TRIM(cd_op) == 'MAX' ) THEN
         ! Find the maximum thickness in each parent grid subset
         IF ( PRESENT(p_pfield3d_1) )  THEN
            zpfield3d(:,:,:) = p_pfield3d_1(:,:,:)
            ze3(:,:,:) = 0.0
            ze1e2(:,:) = 0.0
         ELSE 
            WRITE(numout,*) 'crsfun_TW. MAX only 3D arrays supported' 
         ENDIF 
      ELSEIF ( TRIM(cd_op) == 'VOL' ) THEN
         IF ( PRESENT(p_pfield3d_1) ) THEN
            ze3(:,:,:)       = p_pfield3d_1(:,:,:)
            IF ( PRESENT(p_pfield3d_2) ) THEN
!              ! Prep to calculate a volume-averaged mean
               zpfield3d(:,:,:) = p_pfield3d_2(:,:,:)
               ze1e2(:,:)       = p_e1e2t(:,:)     
            ELSE
               WRITE(numout,*) 'crsfun_TW. WARNING. Supply both e3t and the field for volume-averaged field'
            ENDIF
         ELSE
            WRITE(numout,*) 'crsfun_TW. VOL only 3D arrays supported'  
         ENDIF
      ELSEIF ( TRIM(cd_op) == 'ARE' ) THEN
         ze1e2(:,:) = p_e1e2t(:,:)
         ze3(:,:,:) = 1.0
         IF ( PRESENT(p_pfield3d_2) ) THEN
            ! Prep to do the area-weighted average of (3D) W
            zpfield3d(:,:,:) = p_pfield3d_2(:,:,:)
         ELSEIF ( PRESENT(p_pfield2d) ) THEN
            ! Prep to do the area-weighted average of some 2D quantity 
            zpfield2d(:,:) = p_pfield2d(:,:) 
            ijpk=1
         ENDIF
      ELSEIF ( TRIM(cd_op) == 'MIN' ) THEN
         IF ( PRESENT(p_pfield3d_1) ) THEN
            ! Prep to do get the minimum diffusion on the top face
            zpfield3d(:,:,:) = p_pfield3d_1(:,:,:)
            ze3(:,:,:) = 0.0
            ze1e2(:,:) = 0.0
         ELSE
            WRITE(numout,*) 'crsfun_TW. MIN Only 3D arrays supported' 
         ENDIF
      ELSEIF ( TRIM(cd_op) == 'SUM' ) THEN
         ze1e2(:,:) = p_e1e2t(:,:)
         zpmask(:,:,:) = p_ptmask(:,:,:)
         ze3(:,:,:) = 1.0
        IF ( PRESENT(p_pfield3d_1) ) THEN
            IF ( PRESENT(p_pfield3d_2) ) THEN
!              ! Prep to calculate a volume-weighted sum
               zpfield3d(:,:,:) = p_pfield3d_2(:,:,:)
               ze3(:,:,:) = p_pfield3d_1(:,:,:)
            ELSE
               ! Prep to do the area-weighted sum of (3D) W
               zpfield3d(:,:,:) = p_pfield3d_1(:,:,:)
            ENDIF
         ELSEIF ( PRESENT(p_pfield2d) ) THEN
            ! Prep to do the area-weighted sum of some 2D quantity 
            zpfield2d(:,:) = p_pfield2d(:,:) 
            ijpk=1
         ENDIF 
      ELSE
         WRITE(numout,*) 'crsfun_TW. valid cd_op are MAX, MIN, ARE, VOL, SUM' 
      ENDIF

      ! Determine output 
      DO jk = 1, ijpk    

         IF ( ijpk == jpk ) zpfield2d(:,:) = zpfield3d(:,:,jk) 
         zcfield2d(:,:) = 0.0 

           DO ji = 2, nlei_crs
               ijie = mie_crs(ji)
               ijis = mis_crs(ji)

             DO jj = njstart, njend
                  ijje = mje_crs(jj) 
                  ijjs = mjs_crs(jj)                   

                  ! First determine weighted sums 
                  IF ( TRIM(cd_op) == 'SUM' .OR. TRIM(cd_op) == 'ARE' .OR. TRIM(cd_op) == 'VOL' ) THEN
                     ! Area- or volume- weighted sum
                     ! Accumulate to sum in parent grid subset 
                    DO jii = ijis, ijie
                        DO jjj = ijjs, ijje
                           zcfield2d(ji,jj) = zcfield2d(ji,jj) &
                              &              + ( zpfield2d(jii,jjj)      &
                              &                  *   ze1e2(jii,jjj)      &
                              &                  *     ze3(jii,jjj,jk)   &
                              &                  *  zpmask(jii,jjj,jk) ) 

                        ENDDO
                     ENDDO

                  ENDIF

                  ! Calculate weighted average if desired
                  IF ( TRIM(cd_op) == 'ARE' .OR. TRIM(cd_op) == 'VOL' ) THEN

                     ! Area- or volume- weighted mean
                     ! Sum first parent grid subset  
                     zdAm = 0.0
                     DO jii = ijis, ijie
                        DO jjj = ijjs, ijje
                           zdAm = zdAm + (   ze1e2(jii,jjj)               &
                              &           *    ze3(jii,jjj,jk)            &
                              &           * zpmask(jii,jjj,jk) )
                        ENDDO
                     ENDDO
                     
                     IF ( zdAm /= 0 )  zcfield2d(ji,jj) = zcfield2d(ji,jj) / zdAm

                  ENDIF


                  IF ( TRIM(cd_op) == 'MAX' ) THEN
                     ! Find max in parent grid subset 
                     DO jii = ijis, ijie
                        DO jjj = ijjs, ijje
                           zcfield2d(ji,jj) = MAX( zcfield2d(ji,jj), zpfield3d(jii,jjj,jk)*zpmask(jii,jjj,jk) ) 
                        ENDDO
                     ENDDO
                  ENDIF

                  IF ( TRIM(cd_op) == 'MIN' ) THEN
                     ! Find min in parent grid subset  
                     DO jii = ijis, ijie
                        DO jjj = ijjs, ijje
                           IF ( zpmask(jii,jjj,jk) == 1 ) THEN
                              zcfield2d(ji,jj) = MIN( zcfield2d(ji,jj), zpfield3d(jii,jjj,jk) ) 
                           ENDIF
                        ENDDO
                     ENDDO
                  ENDIF

               ENDDO
            ENDDO

            IF ( ijpk == 1 ) THEN
               IF ( PRESENT(p_cfield2d) ) p_cfield2d(:,:) = zcfield2d(:,:) * zcmask(:,:,jk)
            ELSE
               IF ( PRESENT(p_cfield3d) ) p_cfield3d(:,:,jk) = zcfield2d(:,:) * zcmask(:,:,jk)
            ENDIF    

      ENDDO


! Retroactively add back the boundary halo cells.

         IF ( ijpk == 1 ) THEN
            IF ( PRESENT(p_cfield2d) ) CALL crs_lbc_lnk( p_cfield2d(:,:),cd_type,zsgn )
         ELSE
            IF ( PRESENT(p_cfield3d) ) THEN
               CALL crs_lbc_lnk( p_cfield3d(:,:,:),cd_type,zsgn )
            ENDIF
         ENDIF
     
      CALL wrk_dealloc(jpi    , jpj         , ze1e2, zpfield2d )
      CALL wrk_dealloc(jpi    , jpj    , jpk,  ze3 , zpfield3d, zpmask )
      CALL wrk_dealloc(jpi_crs, jpj_crs,      zcfield2d )
      CALL wrk_dealloc(jpi_crs, jpj_crs, jpk, zcfield3d,  zcmask )
 

   END SUBROUTINE crsfun_TW


   SUBROUTINE crs_dom_e3_max( p_e3, cd_type, p_mask, p_e3_crs)
   
   
      !!  Arguments
      CHARACTER(len=1),                         INTENT(in) :: cd_type      ! grid type T, W ( U, V, F)
      REAL(wp), DIMENSION(jpi,jpj,jpk),         INTENT(in) :: p_mask       ! Parent grid T mask
      REAL(wp), DIMENSION(jpi,jpj,jpk),         INTENT(in) :: p_e3         ! 3D tracer T or W on parent grid
      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(inout):: p_e3_crs ! Coarse grid box east or north face quantity 

      !! Local variables
      INTEGER ::  ji, jj, jk                   ! dummy loop indices
      INTEGER :: ijie,ijis,ijje,ijjs,jii,jjj
      !!----------------------------------------------------------------  
      ! Initialize

      SELECT CASE ( cd_type )
      
         CASE ('T')
         
            DO jk = 1 , jpk
              
               DO ji = 2, nlei_crs        ! ji = 1 et jpi_crs definit par cyclique est-ouest et pivot T
                   ijie = mie_crs(ji)
                   ijis = mis_crs(ji)

                   DO jj =  njstart, njend  ! jj = jpj_crs definit par pivot T 
                      ijje = mje_crs(jj) 
                      ijjs = mjs_crs(jj)  
                      
                      DO jii = ijis, ijie
                         DO jjj = ijjs, ijje
                            p_e3_crs(ji,jj,jk) = max( p_e3_crs(ji,jj,jk), p_e3(jii,jjj,jk) * p_mask(jii,jjj,jk)  )
                         ENDDO
                      ENDDO
                   ENDDO
               ENDDO
            ENDDO
 
         CASE ('W')
         
            DO jk = 2 , jpk
            
                                          ! ji = 1 et jpi_crs definit par cyclique est-ouest et pivot T
               DO ji = 2, nlei_crs
                   ijie = mie_crs(ji)
                   ijis = mis_crs(ji)

                   DO jj = njstart, njend ! jj = jpj_crs definit par pivot T 
                      ijje = mje_crs(jj) 
                      ijjs = mjs_crs(jj)  
                   
                      DO jii = ijis, ijie
                         DO jjj = ijjs, ijje
                            p_e3_crs(ji,jj,jk) = max( p_e3_crs(ji,jj,jk), p_e3(jii,jjj,jk) * p_mask(jii,jjj,jk-1)  )
                         ENDDO
                      ENDDO
                   ENDDO
               ENDDO
            ENDDO
             
            jk = 1                                           ! cas particulier car zpmask(jii,jjj,0) n'existe pas      
          
            DO ji = 2, nlei_crs
               ijie = mie_crs(ji)
               ijis = mis_crs(ji)

               DO jj = njstart, njend            
                  ijje = mje_crs(jj) 
                  ijjs = mjs_crs(jj)  
                   
                  DO jii = ijis, ijie
                     DO jjj = ijjs, ijje
                        p_e3_crs(ji,jj,jk) = max( p_e3_crs(ji,jj,jk), p_e3(jii,jjj,jk) * p_mask(jii,jjj,jk)  )
                     ENDDO
                  ENDDO
               ENDDO
            ENDDO  
            
         END SELECT 
                  
         CALL crs_lbc_lnk( p_e3_crs(:,:,:),cd_type, 1.0, pval=1.0 )  
     
         WRITE(numout,*) 'crs_e3_max : end of subroutine '
 

   END SUBROUTINE crs_dom_e3_max


   SUBROUTINE crs_dom_sfc( p_e1, p_e2, p_e3, cd_type, p_mask, p_surf_crs, p_surf_crs_msk )

      !!  Arguments
      CHARACTER(len=1),                 INTENT(in) :: cd_type      ! grid type T, W ( U, V, F)
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in) :: p_mask       ! Parent grid T mask
      REAL(wp), DIMENSION(jpi,jpj)    , INTENT(in) :: p_e1, p_e2         ! 3D tracer T or W on parent grid
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(in) :: p_e3         ! 3D tracer T or W on parent grid
      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(inout):: p_surf_crs ! Coarse grid box east or north face quantity 
      REAL(wp), DIMENSION(jpi_crs,jpj_crs,jpk), INTENT(inout):: p_surf_crs_msk ! Coarse grid box east or north face quantity 

      !! Local variables
      INTEGER  ::  ji, jj, jk                   ! dummy loop indices
      INTEGER  :: ijie,ijis,ijje,ijjs,ijpk,jii,jjj
      REAL(wp) :: zsfc
      !!----------------------------------------------------------------  
      ! Initialize

      p_surf_crs    (:,:,:) = 0._wp
      p_surf_crs_msk(:,:,:) = 0._wp
      !
      SELECT CASE ( cd_type )
      
         CASE ('W')

            DO jk = 2 , jpk
                DO ji = 2, nlei_crs
                   ijie = mie_crs(ji)
                   ijis = mis_crs(ji)
                   DO jj = njstart, njend
                      ijje = mje_crs(jj) 
                      ijjs = mjs_crs(jj)
                      DO jii = ijis, ijie
                         DO jjj = ijjs, ijje
                            zsfc = p_e1(jii,jjj) * p_e2(jii,jjj)
                            p_surf_crs(ji,jj,jk)     = p_surf_crs(ji,jj,jk) + zsfc
                            p_surf_crs_msk(ji,jj,jk) = p_surf_crs_msk(ji,jj,jk) + zsfc * p_mask(jii,jjj,jk-1)               
                         ENDDO
                      ENDDO
                      IF( njstart == 1 ) p_surf_crs(ji,jj,jk) = p_surf_crs(ji,jj,jk) * nn_facty
                   ENDDO                  
               ENDDO
            ENDDO
                        
            ! surface   ; ji = 1 et jpi_crs definit par cyclique est-ouest et pivot T
            DO ji = 2, nlei_crs
               ijie = mie_crs(ji)
               ijis = mis_crs(ji)
               DO jj = njstart, njend                           
                  ijje = mje_crs(jj) 
                  ijjs = mjs_crs(jj)
                  DO jii = ijis, ijie 
                     DO jjj = ijjs, ijje
                        zsfc = p_e1(jii,jjj) * p_e2(jii,jjj)
                        p_surf_crs(ji,jj,1)     = p_surf_crs(ji,jj,1) + zsfc
                        p_surf_crs_msk(ji,jj,1) = p_surf_crs_msk(ji,jj,1) * zsfc * p_mask(jii,jjj,1)               
                     ENDDO  
                  ENDDO
                  IF( njstart == 1 ) p_surf_crs(ji,jj,1) = p_surf_crs(ji,jj,1) * nn_facty               
               ENDDO
            ENDDO
            
       CASE ('U')
       
          DO jk = 1 , jpk
                                             ! ji = 1 et jpi_crs definit par cyclique est-ouest et pivot T
             DO ji = 2, nlei_crs
                ijie = mie_crs(ji)
                ijis = mis_crs(ji)
                DO jj = njstart, njend 
                   ijje = mje_crs(jj)  
                   ijjs = mjs_crs(jj)
                   DO jii = ijis, ijie
                      DO jjj = ijjs, ijje   
                         zsfc =  p_e3(jii,jjj,jk) * p_e2(jii,jjj)  
                         p_surf_crs(ji,jj,jk)     = p_surf_crs(ji,jj,jk) + zsfc
                         p_surf_crs_msk(ji,jj,jk) = p_surf_crs_msk(ji,jj,jk) + zsfc * p_mask(jii,jjj,jk)               
                      ENDDO
                   ENDDO
                  IF( njstart == 1 ) p_surf_crs(ji,jj,jk) = p_surf_crs(ji,jj,jk) * nn_facty
               ENDDO
            ENDDO
         ENDDO
       
      CASE ('V')
          
         DO jk = 1 , ijpk
                                                                   ! ji = 1 et jpi_crs definit par cyclique est-ouest et pivot T
            DO ji = 2, nlei_crs
               ijie = mie_crs(ji)
               ijis = mis_crs(ji)
               DO jj = njstart, njend                  ! jj = jpj_crs definit par pivot T 
                  ijje = mje_crs(jj)   
                  ijjs = mjs_crs(jj)  
                  DO jii = ijis, ijie
                     DO jjj = ijjs, ijje
                        zsfc = p_e3(jii,jjj,jk) * p_e1(jii,jjj) 
                        p_surf_crs(ji,jj,jk)     = p_surf_crs(ji,jj,jk) + zsfc
                        p_surf_crs_msk(ji,jj,jk) = p_surf_crs_msk(ji,jj,jk) + zsfc * p_mask(jii,jjj,jk)   
                     ENDDO
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      END SELECT
 
      
      CALL crs_lbc_lnk( p_surf_crs    (:,:,:), cd_type, 1.0, pval=1.0 )
      CALL crs_lbc_lnk( p_surf_crs_msk(:,:,:), cd_type, 1.0, pval=1.0 )


   END SUBROUTINE crs_dom_sfc
   
   SUBROUTINE crs_dom_def
      !!----------------------------------------------------------------
      !!               *** SUBROUTINE crs_dom_def ***
      !! ** Purpose :  Three applications.
      !!               1) Define global domain indice of the croasening grid
      !!               2) Define local domain indice of the croasening grid
      !!               3) Define the processor domain indice for a croasening grid
      !!----------------------------------------------------------------
      !! 
      !!  local variables
   
      INTEGER  :: ji,jj,jk,ijjgloT,ijis,ijie,ijjs,ijje,jn      ! dummy indices
      INTEGER  :: ierr                                ! allocation error status
   
     ! 1.a. Define global domain indices
      jpiglo_crs   = INT( (jpiglo - 2) / nn_factx ) + 2
      jpjglo_crs   = INT( (jpjglo - 2) / nn_facty ) + 2  ! the -2 removes j=1, j=jpj
      jpiglo_crsm1 = jpiglo_crs - 1
      jpjglo_crsm1 = jpjglo_crs - 1  

     ! 1.b. Define local domain indices
      jpi_crs = ( jpiglo_crs-2 * jpreci + (jpni-1) ) / jpni + 2*jpreci
      jpj_crs = ( jpjglo_crs-2 * jprecj + (jpnj-1) ) / jpnj + 2*jprecj
       
      jpi_crsm1   = jpi_crs - 1
      jpj_crsm1   = jpj_crs - 1
      nperio_crs  = jperio
      npolj_crs   = npolj
      
      ierr = crs_dom_alloc()          ! allocate most coarse grid arrays
     ! 2.a Define processor domain
      IF( .NOT. lk_mpp ) THEN
         nimpp_crs  = 1
         njmpp_crs  = 1
         nlci_crs   = jpi_crs
         nlcj_crs   = jpj_crs
         nldi_crs   = 1
         nldj_crs   = 1
         nlei_crs   = jpi_crs
         nlej_crs   = jpj_crs

      ELSE
         ! Initialisation of most local variables -
         nimpp_crs  = 1
         njmpp_crs  = 1
         nlci_crs   = jpi_crs
         nlcj_crs   = jpj_crs
         nldi_crs   = 1
         nldj_crs   = 1
         nlei_crs   = jpi_crs
         nlej_crs   = jpj_crs
         
        !!! Calculs suivant une découpage en j
     
        DO jn = 1, jpnij, jpni
         
           IF( jn < (jpnij-jpni + 1)) THEN
              nlejt_crs(jn) = AINT( REAL( ( jpjglo - (njmppt(jn     ) - 1) ) / nn_facty, wp ) ) &
                       &    - AINT( REAL( ( jpjglo - (njmppt(jn+jpni) - 1) ) / nn_facty, wp ) )
           ELSE                                             
              nlejt_crs(jn) = AINT( REAL(  nlejt(jn) / nn_facty, wp ) ) + 1            
           ENDIF
           
           SELECT CASE( ibonjt(jn) )
           
              CASE ( -1 )
                IF( MOD( jpjglo - njmppt(jn), nn_facty) > 0 )    nlejt_crs(jn) = nlejt_crs(jn) + 1
                nlcjt_crs(jn) = nlejt_crs(jn) + jprecj
                nldjt_crs(jn) = nldjt(jn)
              
              CASE ( 0 )
              
                nlejt_crs(jn) = nlejt_crs(jn) + jprecj
                nlcjt_crs(jn) = nlejt_crs(jn) + jprecj
                nldjt_crs(jn) = nldjt(jn)
                
              CASE ( 1, 2 )
              
                nlejt_crs(jn) = nlejt_crs(jn) + jprecj
                nlcjt_crs(jn) = nlejt_crs(jn)
                nldjt_crs(jn) = nldjt(jn)
                
              CASE DEFAULT
                 STOP
           END SELECT
         
           njmppt_crs(jn) = ANINT(REAL((njmppt(jn) + 1 + MOD( jpjglo - njmppt(jn) + 1, nn_facty )) / nn_facty, wp ) )
           
           DO jj = jn + 1, jn + jpni - 1
              nlejt_crs(jj) = nlejt_crs(jn) 
              nlcjt_crs(jj) = nlcjt_crs(jn)
              nldjt_crs(jj) = nldjt_crs(jn)
              njmppt_crs(jj)= njmppt_crs(jn)
           ENDDO
        ENDDO 
                 WRITE(numout,*) ' njmppt_crs', njmppt_crs
         nlej_crs  = nlejt_crs(nproc + 1) 
         nlcj_crs  = nlcjt_crs(nproc + 1)
         nldj_crs  = nldjt_crs(nproc + 1)
         njmpp_crs = njmppt_crs(nproc + 1)



         !!!! Calcul suivant un decoupage en i
           DO jn = 1, jpni
              IF( jn < jpni ) THEN
         
                 nleit_crs(jn) = AINT( REAL( ( jpiglo - (nimppt(jn  ) - 1) ) / nn_factx, wp ) ) &
                         &     - AINT( REAL( ( jpiglo - (nimppt(jn+1) - 1) ) / nn_factx, wp ) )
              ELSE
                 nleit_crs(jn) = AINT( REAL( ( jpiglo - (nimppt(jn  ) - 1) ) / nn_factx, wp ) )
              ENDIF
              SELECT CASE( ibonit(jn) )
           
                 CASE ( -1 )
                 
                   nleit_crs(jn) = nleit_crs(jn) + jpreci           
                   nlcit_crs(jn) = nleit_crs(jn) + jpreci
                   nldit_crs(jn) = nldit(jn) 
              
                 CASE ( 0 )
              
                   nleit_crs(jn) = nleit_crs(jn) + jpreci
                   nlcit_crs(jn) = nleit_crs(jn) + jpreci
                   nldit_crs(jn) = nldit(jn) 
                
                 CASE ( 1, 2 )
                 
                   IF( MOD( jpiglo - nimppt(jn), nn_factx) > 0 )    nleit_crs(jn) = nleit_crs(jn) + 1
                   nleit_crs(jn) = nleit_crs(jn) + jpreci
                   nlcit_crs(jn) = nleit_crs(jn)
                   nldit_crs(jn) = nldit(jn) 
                
                 CASE DEFAULT
                    STOP
              END SELECT

              nimppt_crs(jn) =ANINT( REAL( (nimppt(jn) + 1 ) / nn_factx, wp ) ) + 1
              DO jj = jn+jpni , jpnij, jpni
                 nleit_crs(jj) = nleit_crs(jn) 
                 nlcit_crs(jj) = nlcit_crs(jn)
                 nldit_crs(jj) = nldit_crs(jn)
                 nimppt_crs(jj)= nimppt_crs(jn)
              ENDDO
           ENDDO 
        
         nlei_crs  = nleit_crs(nproc + 1) 
         nlci_crs  = nlcit_crs(nproc + 1)
         nldi_crs  = nldit_crs(nproc + 1)
         nimpp_crs = nimppt_crs(nproc + 1)


         
         !!! rajouter la condition stop
         IF( jpizoom /= 1 .OR. jpjzoom /= 1)    STOP 
         DO ji = 1, jpi_crs
            mig_crs(ji) = ji + jpizoom - 1 + nimpp_crs - 1
         ENDDO
         DO jj = 1, jpj_crs
            mjg_crs(jj) = jj + jpjzoom - 1 + njmpp_crs - 1!
         ENDDO

         
          
      ENDIF
      
      !                         Save the parent grid information
      jpi_full    = jpi
      jpj_full    = jpj
      jpim1_full  = jpim1
      jpjm1_full  = jpjm1
      nperio_full = nperio

      npolj_full  = npolj
      jpiglo_full = jpiglo
      jpjglo_full = jpjglo

      nlcj_full   = nlcj
      nlci_full   = nlci
      nldi_full   = nldi
      nldj_full   = nldj
      nlei_full   = nlei
      nlej_full   = nlej
      nimpp_full  = nimpp     
      njmpp_full  = njmpp
      
      nlcit_full(:)  = nlcit(:)
      nldit_full(:)  = nldit(:)
      nleit_full(:)  = nleit(:)
      nimppt_full(:) = nimppt(:)
      nlcjt_full(:)  = nlcjt(:)
      nldjt_full(:)  = nldjt(:)
      nlejt_full(:)  = nlejt(:)
      njmppt_full(:) = njmppt(:)
      
      CALL dom_grid_crs  !swich de grille
     

      IF (lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'crs_init : coarse grid dimensions'
         WRITE(numout,*) '~~~~~~~   coarse domain global j-dimension           jpjglo = ', jpjglo
         WRITE(numout,*) '~~~~~~~   coarse domain global i-dimension           jpiglo = ', jpiglo
         WRITE(numout,*) '~~~~~~~   coarse domain local  i-dimension              jpi = ', jpi
         WRITE(numout,*) '~~~~~~~   coarse domain local  j-dimension              jpj = ', jpj
         WRITE(numout,*)
         WRITE(numout,*) ' nproc  = '     , nproc
         WRITE(numout,*) ' nlci   = '     , nlci
         WRITE(numout,*) ' nlcj   = '     , nlcj
         WRITE(numout,*) ' nldi   = '     , nldi
         WRITE(numout,*) ' nldj   = '     , nldj
         WRITE(numout,*) ' nlei   = '     , nlei
         WRITE(numout,*) ' nlej   = '     , nlej
         WRITE(numout,*) ' nlei_full='    , nlei_full
         WRITE(numout,*) ' nldi_full='    , nldi_full
         WRITE(numout,*) ' nimpp  = '     , nimpp
         WRITE(numout,*) ' njmpp  = '     , njmpp
         WRITE(numout,*) ' njmpp_full  = ', njmpp_full
         WRITE(numout,*) ' nreci'         , nreci 
         WRITE(numout,*) ' nlejt'         , nlejt
         WRITE(numout,*) ' nldjt'         , nldjt
         WRITE(numout,*) ' nlcjt'         , nlcjt
         WRITE(numout,*) ' njmppt'        , njmppt
         WRITE(numout,*) ' nleit'         , nleit
         WRITE(numout,*) ' nldit'         , nldit
         WRITE(numout,*) ' nlcit'         , nlcit         
         WRITE(numout,*) ' nimppt'        , nimppt
         WRITE(numout,*) ' nleit_full'    , nleit_full
         WRITE(numout,*)
      ENDIF
      

      CALL dom_grid_glo
      
      mxbinctr   = INT( nn_factx * 0.5 )
      mybinctr   = INT( nn_facty * 0.5 )

      nrestx = MOD( nn_factx, 2 )   ! check if even- or odd- numbered reduction factor
      nresty = MOD( nn_facty, 2 )

      IF ( nrestx == 0 ) THEN
         mxbinctr = mxbinctr - 1
      ENDIF

      IF ( nresty == 0 ) THEN
         mybinctr = mybinctr - 1
         IF ( jperio == 3 .OR. jperio == 4 )  nperio_crs = jperio + 2
         IF ( jperio == 5 .OR. jperio == 6 )  nperio_crs = jperio - 2 

         IF ( npolj == 3 ) npolj_crs = 5
         IF ( npolj == 5 ) npolj_crs = 3
      ENDIF     
      
      rfactxy = nn_factx * nn_facty
      
      ! 2.b. Set up bins for coarse grid, horizontal only.
      ierr = crs_dom_alloc2()
      mis2_crs(:) = 0; mie2_crs(:) = 0
      mjs2_crs(:) = 0; mje2_crs(:) = 0

      
      SELECT CASE ( cn_binref )

      CASE ( 'NORTH' ) 

         SELECT CASE ( nperio )
     
 
        CASE ( 0, 1, 3, 4 )    !   3, 4 : T-Pivot at North Fold
        
            DO ji = 2, jpiglo_crsm1
               ijie = (ji*nn_factx)-nn_factx   !cc
               ijis = ijie-nn_factx+1
               mis2_crs(ji) = ijis
               mie2_crs(ji) = ijie
            ENDDO
            IF ( jpiglo - 1 - mie2_crs(jpiglo_crsm1) <= nn_factx ) mie2_crs(jpiglo_crsm1)  = jpiglo-2  

            ! Handle first the northernmost bin
            IF ( nn_facty == 2 ) THEN   ;    ijjgloT = jpjglo - 1 
            ELSE                        ;    ijjgloT = jpjglo
            ENDIF

            DO jj = 2, jpjglo_crsm1
                ijje = ijjgloT-nn_facty*(jj-2)
                ijjs = ijje-nn_facty+1                   
                mjs2_crs(jpjglo_crs-jj+1) = ijjs
                mje2_crs(jpjglo_crs-jj+1) = ijje
            ENDDO

         CASE ( 2 ) 
            WRITE(numout,*)  'crs_init, jperio=2 not supported' 
        
         CASE ( 5, 6 )    ! F-pivot at North Fold

            DO ji = 2, jpiglo_crsm1
               ijie = (ji*nn_factx)-nn_factx 
               ijis = ijie-nn_factx+1
               mis2_crs(ji) = ijis
               mie2_crs(ji) = ijie
            ENDDO
            IF ( jpiglo - 1 - mie2_crs(jpiglo_crsm1) <= nn_factx ) mie_crs(jpiglo_crsm1)  = jpiglo-2 

            ! Treat the northernmost bin separately.
            jj = 2
            ijje = jpj-nn_facty*(jj-2)
            IF ( nn_facty == 3 ) THEN   ;  ijjs = ijje - 1 
            ELSE                        ;  ijjs = ijje - nn_facty + 1
            ENDIF
            mjs2_crs(jpj_crs-jj+1) = ijjs
            mje2_crs(jpj_crs-jj+1) = ijje

            ! Now bin the rest, any remainder at the south is lumped in the southern bin
            DO jj = 3, jpjglo_crsm1
                ijje = jpjglo-nn_facty*(jj-2)
                ijjs = ijje-nn_facty+1                  
                IF ( ijjs <= nn_facty )   ijjs = 2
                mjs2_crs(jpj_crs-jj+1) = ijjs
                mje2_crs(jpj_crs-jj+1) = ijje
            ENDDO

         CASE DEFAULT
            WRITE(numout,*) 'crs_init. Only jperio = 0, 1, 3, 4, 5, 6 supported' 
 
         END SELECT

      CASE ( 'EQUAT' )
         WRITE(numout,*) 'crs_init.  Equator-centered bins option not yet available' 

      END SELECT


        ! Pad the boundaries, do not know if it is necessary
         mis2_crs(1) = 1            ; mis2_crs(jpiglo_crs) = mie2_crs(jpiglo_crs - 1) + 1   
         mie2_crs(1) = nn_factx     ; mie2_crs(jpiglo_crs) = jpiglo                         
         mje2_crs(1) = mjs2_crs(2)-1; mje2_crs(jpjglo_crs) = jpjglo
         mjs2_crs(1) = 1            ; mjs2_crs(jpjglo_crs) = mje2_crs(jpjglo_crs) - nn_facty + 1 
 

         
 
      IF( .NOT. lk_mpp ) THEN     
         njstart = 1     ;    njend =  jpj_crsm1
         mis_crs(:) = mis2_crs(:) 
         mie_crs(:) = mie2_crs(:)
         mjs_crs(:) = mjs2_crs(:) 
         mje_crs(:) = mje2_crs(:)
      ELSE
         !
         IF( nldj==1 )  THEN  ;   njstart = 1
         ELSE                 ;   njstart = 2
         ENDIF
         !
         IF(nlcj == nlej) THEN
            njend = nlej_crs - 1
         ELSE
            njend = nlej_crs
         ENDIF
         
         DO jj = 1, nlej_crs
            mjs_crs(jj) = mjs2_crs(mjg_crs(jj)) - njmpp + 1
            mje_crs(jj) = mje2_crs(mjg_crs(jj)) - njmpp + 1
         ENDDO
         DO ji = 1, nlei_crs
            mis_crs(ji) = mis2_crs(mig_crs(ji)) - nimpp + 1
            mie_crs(ji) = mie2_crs(mig_crs(ji)) - nimpp + 1
         ENDDO

         !
      ENDIF
         WRITE(numout,*) 'crs_init. coarse grid bounds on parent grid'
         WRITE(numout,*) 'mjg_crs='  , mjg_crs
         WRITE(numout,*) 'mig_crs='  , mig_crs
         WRITE(numout,*) 'nimpp_crs=', nimpp_crs
         WRITE(numout,*) 'njmpp_crs=', njmpp_crs
         WRITE(numout,*) 'njend='    , njend
         WRITE(numout,*) 'mis_crs='  , mis_crs
         WRITE(numout,*) 'mie_crs='  , mie_crs
         WRITE(numout,*) 'mjs_crs='  , mjs_crs
         WRITE(numout,*) 'mje_crs='  , mje_crs
   
   
   END SUBROUTINE crs_dom_def
   
   SUBROUTINE crs_dom_bat
    !!----------------------------------------------------------------
      !!               *** SUBROUTINE crs_dom_bat ***
      !! ** Purpose :  coarsenig bathy
      !!----------------------------------------------------------------
      !! 
      !!  local variables

   
   INTEGER  :: ji,jj,jk      ! dummy indices
   REAL(wp), DIMENSION(:,:)  , POINTER :: zmbk
   
   CALL wrk_alloc( jpi_crs, jpj_crs, zmbk )
   
      mbathy_crs(:,:) = jpkm1
      mbkt_crs(:,:) = 1
      mbku_crs(:,:) = 1
      mbkv_crs(:,:) = 1


      DO jj = 1, jpj_crs
         DO ji = 1, jpi_crs
            jk = 0
            DO WHILE( tmask_crs(ji,jj,jk+1) > 0.)
               jk = jk + 1
            ENDDO
            mbathy_crs(ji,jj) = float( jk )
         ENDDO
      ENDDO
     
      CALL wrk_alloc( jpi_crs, jpj_crs, zmbk )

      zmbk(:,:) = 0.0
      zmbk(:,:) = REAL( mbathy_crs(:,:), wp ) ;   CALL crs_lbc_lnk(zmbk,'T',1.0)   ;   mbathy_crs(:,:) = INT( zmbk(:,:) )


      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) '    crsini : mbkt is ocean bottom k-index of T-, U-, V- and W-levels '
      IF(lwp) WRITE(numout,*) '    ~~~~~~~~~~~~~'
      !
      mbkt_crs(:,:) = MAX( mbathy_crs(:,:) , 1 )    ! bottom k-index of T-level (=1 over land)
      !                                     ! bottom k-index of W-level = mbkt+1

      DO jj = 1, jpj_crsm1                      ! bottom k-index of u- (v-) level
         DO ji = 1, jpi_crsm1
            mbku_crs(ji,jj) = MIN(  mbkt_crs(ji+1,jj  ) , mbkt_crs(ji,jj)  )
            mbkv_crs(ji,jj) = MIN(  mbkt_crs(ji  ,jj+1) , mbkt_crs(ji,jj)  )
         END DO
      END DO

      WRITE(numout,*) 'crsini. Set mbku, mkbv'

      ! convert into REAL to use lbc_lnk ; impose a min value of 1 as a zero can be set in lbclnk
  
      zmbk(:,:) = 1.e0;    
      zmbk(:,:) = REAL( mbku_crs(:,:), wp )   ;   CALL crs_lbc_lnk(zmbk,'U',1.0) ; mbku_crs  (:,:) = MAX( INT( zmbk(:,:) ), 1 ) 
      zmbk(:,:) = REAL( mbkv_crs(:,:), wp )   ;   CALL crs_lbc_lnk(zmbk,'V',1.0) ; mbkv_crs  (:,:) = MAX( INT( zmbk(:,:) ), 1 ) 
      !
      CALL wrk_dealloc( jpi_crs, jpj_crs, zmbk )
      WRITE(numout,*) 'crs_init   = finished section 3d.1 jpi=', jpi, 'jpj=',jpj, ' jpk=', jpk

   END SUBROUTINE crs_dom_bat


END MODULE crsdom
